@@ -91,6 +91,10 @@
   (unless (fboundp 'eieio-slot-descriptor-name)
     (defsubst eieio-slot-descriptor-name (slot) (aref slot 0))))
 
+(eval-and-compile
+  (unless (fboundp 'cl--slot-descriptor-initform)
+    (defsubst cl--slot-descriptor-initform (slot) (aref slot 1))))
+
 (eval-when-compile
   (unless (fboundp 'cl--slot-descriptor-type)
     (defsubst cl--slot-descriptor-type (slot) (aref slot 2))))
@@ -516,15 +520,13 @@ The optional POS argument indicates current byte index of the field (used by
          (setq condition (car i))
          (setq name-list (cdr i))
          (setq flag nil)
-         (if (symbolp condition)
-             (setq condition (symbol-value condition))
-           (when (and (listp condition) (eq 'logior (car condition)))
-             (setq condition (apply #'logior (cdr condition)))))
          (cl-assert (or (integerp condition) (listp condition)))
          (if (integerp condition)
              (setq flag (/= 0 (logand expression condition)))
-           (while (and (not flag) condition)
-             (setq flag (or flag (= expression (pop condition))))))
+           (if (eq 'logior (car condition))
+               (setq flag (/= 0 (logand expression
+                                        (apply #'logior (cdr condition)))))
+             (setq flag (memq expression condition))))
          (when flag
            (dolist (name name-list)
              (catch 'break
@@ -532,11 +534,13 @@ The optional POS argument indicates current byte index of the field (used by
                  (when (eq name (eieio-slot-descriptor-name slot))
                    (setq slot-type (cl--slot-descriptor-type slot))
                    (throw 'break nil))))
-             (setq result
-                   (vconcat result
-                            (xcb:-marshal-field obj slot-type
-                                                (slot-value obj name)
-                                                (+ pos (length result))))))))
+             (unless (eq slot-type 'xcb:-ignore)
+               (setq result
+                     (vconcat result
+                              (xcb:-marshal-field obj slot-type
+                                                  (slot-value obj name)
+                                                  (+ pos
+                                                     (length result)))))))))
        result))
     ((guard (child-of-class-p type 'xcb:-struct))
      (xcb:marshal value))
@@ -652,13 +656,13 @@ and the second the consumed length."
          (setq condition (car i))
          (setq name-list (cdr i))
          (setq flag nil)
+         (cl-assert (or (integerp condition) (listp condition)))
          (if (integerp condition)
              (setq flag (/= 0 (logand expression condition)))
            (if (eq 'logior (car condition))
                (setq flag (/= 0 (logand expression
                                         (apply #'logior (cdr condition)))))
-             (while (and (not flag) condition)
-               (setq flag (or flag (= expression (pop condition)))))))
+             (setq flag (memq expression condition))))
          (when flag
            (dolist (name name-list)
              (catch 'break
@@ -666,10 +670,12 @@ and the second the consumed length."
                  (when (eq name (eieio-slot-descriptor-name slot))
                    (setq slot-type (cl--slot-descriptor-type slot))
                    (throw 'break nil))))
-             (setq tmp (xcb:-unmarshal-field obj slot-type data offset nil))
-             (setf (slot-value obj name) (car tmp))
-             (setq count (+ count (cadr tmp)))
-             (setq data (substring data (cadr tmp))))))
+             (unless (eq slot-type 'xcb:-ignore)
+               (setq tmp (xcb:-unmarshal-field obj slot-type data offset
+                                               (eieio-oref-default obj name)))
+               (setf (slot-value obj name) (car tmp))
+               (setq count (+ count (cadr tmp)))
+               (setq data (substring data (cadr tmp)))))))
        (list initform count)))
     ((and x (guard (child-of-class-p x 'xcb:-struct)))
      (let* ((struct-obj (make-instance x))
@@ -684,17 +690,11 @@ and the second the consumed length."
   :documentation "X request type.")
 
 (defclass xcb:-reply (xcb:-struct)
-  ((length :initarg :length ;reply length, used in e.g. GetKeyboardMapping
-           :type xcb:-ignore))
+  ((~reply :initform 1 :type xcb:-u1))
   :documentation "X reply type.")
-;;
-(cl-defmethod xcb:unmarshal ((obj xcb:-reply) byte-array)
-  "Fill in fields in a reply OBJ according to its byte-array representation."
-  (cl-call-next-method obj (vconcat (substring byte-array 1 2)
-                                    (substring byte-array 8))))
 
 (defclass xcb:-event (xcb:-struct)
-  nil
+  ((~code :type xcb:-u1))
   :documentation "Event type.")
 ;; Implemented in 'xcb.el'
 (cl-defgeneric xcb:-error-or-event-class->number ((obj xcb:connection) class))
@@ -702,46 +702,48 @@ and the second the consumed length."
 (cl-defmethod xcb:marshal ((obj xcb:-event) connection &optional sequence)
   "Return the byte-array representation of event OBJ.
 
-This method is mainly designed for `xcb:SendEvent', where it's used to generate
-synthetic events. The CONNECTION argument is used to retrieve the event number
-for extensions. If SEQUENCE is non-nil, it is used as the sequence number in
-the synthetic event. Otherwise, 0 is assumed.
+This method is mainly designed for `xcb:SendEvent', where it's used to
+generate synthetic events.  The CONNECTION argument is used to retrieve
+the event number of extensions.  If SEQUENCE is non-nil, it is used as
+the sequence number of the synthetic event (if the event uses sequence
+number); otherwise, 0 is assumed.
 
-Note that this method auto pads the result to 32 bytes, as is always the case."
-  (let ((result (cl-call-next-method obj)))
-    (setq result (vconcat
-                  `[,(xcb:-error-or-event-class->number ;defined in 'xcb.el'
-                      connection (eieio-object-class obj))]
-                  result))
-    (unless (same-class-p obj 'xcb:KeymapNotify)
-      (setq result
-            (vconcat (substring result 0 2)
-                     (funcall (if (slot-value obj '~lsb) #'xcb:-pack-u2-lsb
-                                #'xcb:-pack-u2)
-                              (or sequence 0))
-                     (substring result 2))))
-    (cl-assert (>= 32 (length result)))
-    (setq result (vconcat result (make-vector (- 32 (length result)) 0)))))
-;;
-(cl-defmethod xcb:unmarshal ((obj xcb:-event) byte-array)
-  "Fill in event OBJ according to its byte-array representation BYTE-ARRAY."
-  (cl-call-next-method obj
-                       (if (same-class-p obj 'xcb:KeymapNotify)
-                           (substring byte-array 1) ;strip event code
-                         ;; Strip event code & sequence number
-                         (vconcat (substring byte-array 1 2)
-                                  (substring byte-array 4)))))
+This method auto-pads short results to 32 bytes."
+  (let ((event-number
+         (xcb:-error-or-event-class->number connection
+                                            (eieio-object-class obj)))
+        result)
+    (when (consp event-number)
+      (setq event-number (cdr event-number))
+      (if (= 1 (length event-number))
+          ;; XKB event.
+          (setf (slot-value obj 'xkbType) (aref event-number 0))
+        ;; Generic event.
+        (setf (slot-value obj 'extensions) (aref event-number 0)
+              (slot-value obj 'evtype) (aref event-number 1))))
+    (when (slot-exists-p obj '~sequence)
+      (setf (slot-value obj '~sequence) (or sequence 0)))
+    (setq result (cl-call-next-method obj))
+    (when (> 32 (length result))
+      (setq result (vconcat result (make-vector (- 32 (length result)) 0))))
+    result))
+
+(defclass xcb:-generic-event (xcb:-event)
+  ((~code :initform 35)
+   (~extension :type xcb:CARD8)
+   (~sequence :type xcb:CARD16)
+   (~length :type xcb:CARD32)
+   (~evtype :type xcb:CARD16))
+  :documentation "Generic event type.")
 
 (defclass xcb:-error (xcb:-struct)
-  nil
+  ((~error :initform 0 :type xcb:-u1)
+   (~code :type xcb:-u1)
+   (~sequence :type xcb:CARD16))
   :documentation "X error type.")
-;;
-(cl-defmethod xcb:unmarshal ((obj xcb:-error) byte-array)
-  "Fill in error OBJ according to its byte-array representation BYTE-ARRAY."
-  (cl-call-next-method obj (substring byte-array 4))) ;skip the first 4 bytes
 
 (defclass xcb:-union (xcb:-struct)
-  nil
+  ((~size :initarg :~size :type xcb:-ignore)) ;Size of the largest member.
   :documentation "Union type.")
 ;;
 (cl-defmethod xcb:marshal ((obj xcb:-union))
@@ -749,8 +751,9 @@ Note that this method auto pads the result to 32 bytes, as is always the case."
 
 This result is converted from the first bounded slot."
   (let ((slots (eieio-class-slots (eieio-object-class obj)))
-        result slot type name)
-    (while (and (not result) slots)
+        (size (slot-value obj '~size))
+        result slot type name tmp)
+    (while (and (not result) slots (> size (length result)))
       (setq slot (pop slots))
       (setq type (cl--slot-descriptor-type slot)
             name (eieio-slot-descriptor-name slot))
@@ -760,9 +763,12 @@ This result is converted from the first bounded slot."
                   (and (eq type 'xcb:-list)
                        (not (slot-boundp obj (plist-get (slot-value obj name)
                                                         'name)))))
-        (setq result (xcb:-marshal-field obj
-                                         (cl--slot-descriptor-type slot)
-                                         (slot-value obj name)))))
+        (setq tmp (xcb:-marshal-field obj (cl--slot-descriptor-type slot)
+                                      (slot-value obj name)))
+        (when (> (length tmp) (length result))
+          (setq result tmp))))
+    (when (> size (length result))
+      (setq result (vconcat result (make-vector (- size (length result)) 0))))
     result))
 ;;
 (cl-defmethod xcb:unmarshal ((obj xcb:-union) byte-array &optional ctx)
@@ -770,7 +776,7 @@ This result is converted from the first bounded slot."
 
 The optional argument CTX is for <paramref>."
   (let ((slots (eieio-class-slots (eieio-object-class obj)))
-        slot-name consumed tmp type)
+        slot-name tmp type)
     (dolist (slot slots)
       (setq type (cl--slot-descriptor-type slot))
       (unless (eq type 'xcb:-ignore)
@@ -779,9 +785,8 @@ The optional argument CTX is for <paramref>."
                                         (when (slot-boundp obj slot-name)
                                           (eieio-oref-default obj slot-name))
                                         ctx))
-        (setf (slot-value obj (eieio-slot-descriptor-name slot)) (car tmp))
-        (setq consumed (cadr tmp))))
-    consumed))                          ;consume byte-array only once
+        (setf (slot-value obj (eieio-slot-descriptor-name slot)) (car tmp))))
+    (slot-value obj '~size)))
 
 
 
